<?php

namespace Drupal\sggeoprotection\Controller;

class SggeoprotectionContentController
{
    public $isPRO;
    public $myIP;
    public $params;
    public $myCountryCode;

    function __construct()
    {
        module_load_include('php', 'sggeoprotection', 'classes/geo');
        module_load_include('php', 'sggeoprotection', 'classes/seo_sg_protection');

        \SEO_SG_Protection::CheckBlockLog();

        $this->isPRO = \SEO_SG_Protection::CheckIfPRO();
        $this->myIP = \SEO_SG_Protection::GetMyIP();
        $this->params = \SEO_SG_Protection::Get_Params(array());

        $this->params['frontend_country_list'] = (isset($this->params['frontend_country_list'])) ? json_decode($this->params['frontend_country_list'], true) : '';

        $this->params['backend_country_list'] = (isset($this->params['backend_country_list'])) ? json_decode($this->params['backend_country_list'], true) : '';

        $this->params['redirects'] = (isset($this->params['redirects'])) ? json_decode($this->params['redirects'], true) : '';

        $this->myCountryCode = (filter_var($this->myIP, FILTER_VALIDATE_IP)) ? \SEO_SG_Protection::GetCountryCode($this->myIP) : '';

        $this->myCountry = $this->myCountryCode ? \SEO_SG_Protection::$country_list[$this->myCountryCode] : '';

        if (!$this->isPRO) {
            $data = array();
            $data['custom_status'] = 0;
            \SEO_SG_Protection::Set_Params($data);
            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();
        }

        $this->load_geo_to_sql();
    }

    public function content()
    {
        $variables = array(
            '#theme' => 'sggeoprotection_frontend',
            '#isPro' => $this->isPRO,
            '#rek1' => $GLOBALS['base_url'] . DIRECTORY_SEPARATOR . drupal_get_path('module', 'sggeoprotection') . DIRECTORY_SEPARATOR . 'images' . DIRECTORY_SEPARATOR . 'rek1.png',
            '#rek2' => $GLOBALS['base_url'] . DIRECTORY_SEPARATOR . drupal_get_path('module', 'sggeoprotection') . DIRECTORY_SEPARATOR . 'images' . DIRECTORY_SEPARATOR . 'rek2.png',
            '#rek3' => $GLOBALS['base_url'] . DIRECTORY_SEPARATOR . drupal_get_path('module', 'sggeoprotection') . DIRECTORY_SEPARATOR . 'images' . DIRECTORY_SEPARATOR . 'rek3.png',
            '#antvinst' => \SEO_SG_Protection::CheckAntivirusInstallation(),
            '#country_list' => \SEO_SG_Protection::$country_list,
            '#params' => $this->params,
            '#myIP' => $this->myIP,
            '#myCountry' => $this->myCountry,
            '#country_type_list' => \SEO_SG_Protection::$country_type_list,
            '#myCountryCode' => $this->myCountryCode,
            '#print_icon_data' => array(),
        );


        $action = '';
        if (isset($_REQUEST['action'])) $action = htmlspecialchars(stripslashes(trim($_REQUEST['action'])));

        if ($action == 'EnableDisable_frontend_protection') {
            $params = \SEO_SG_Protection::Get_Params(array('protection_frontend'));
            \SEO_SG_Protection::Set_Params(array('protection_frontend' => round(1 - $params['protection_frontend'])));

            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();

            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
        }

        if ($action == 'Save_frontend_params') {

            $data = array();

            if (isset($_POST['frontend_ip_list'])) $data['frontend_ip_list'] = htmlspecialchars(stripslashes(trim($_POST['frontend_ip_list'])));

            if (isset($_POST['frontend_ip_list_allow'])) $data['frontend_ip_list_allow'] = htmlspecialchars(stripslashes(trim($_POST['frontend_ip_list_allow'])));

            if (isset($_POST['country_list'])) {
                $data['frontend_country_list'] = $_POST['country_list'];
            } else {
                $data['frontend_country_list'] = array();
            }

            if (!$this->isPRO && count($data['frontend_country_list']) > 5) {
                $data['frontend_country_list'] = array_slice($data['frontend_country_list'], 0, 5);
                $message_data = array(
                    'type' => 'info',
                    'header' => 'Free version limits',
                    'message' => 'Limit is 5 countries. Please upgrade. <br><b> For all websites with our <a href="https://www.siteguarding.com/en/drupal-antivirus-website-protection" target="_blank">PRO Antivirus plugin</a>, we provide with free license.</b>',
                    'button_text' => 'Upgrade',
                    'button_url' => 'https://www.siteguarding.com/en/buy-extention/drupal-geo-website-protection?domain=' . urlencode($GLOBALS['base_url']),
                    'help_text' => ''
                );

                $variables['#print_icon_data'] = $message_data;

                //update data
                $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
                $variables['#params']['frontend_country_list'] = (isset($variables['#params']['frontend_country_list'])) ? json_decode($variables['#params']['frontend_country_list'], true) : '';
                //update data

                return $variables;

            }

            $data['frontend_country_list'] = json_encode($data['frontend_country_list']);

            $message_data = array(
                'type' => 'info',
                'header' => '',
                'message' => 'Front-end settings saved',
                'button_text' => '',
                'button_url' => '',
                'help_text' => ''
            );

            $variables['#print_icon_data'] = $message_data;

            \SEO_SG_Protection::Set_Params($data);

            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();

            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
            $variables['#params']['frontend_country_list'] = (isset($variables['#params']['frontend_country_list'])) ? json_decode($variables['#params']['frontend_country_list'], true) : '';
            //update data

        }

        return $variables;
    }

    public function backendprot()
    {
        $variables = array(
            '#theme' => 'sggeoprotection_backend',
            '#params' => $this->params,
            '#isPro' => $this->isPRO,
            '#myIP' => $this->myIP,
            '#myCountry' => $this->myCountry,
            '#myCountryCode' => $this->myCountryCode,
            '#print_icon_data' => array(),
            '#country_list' => \SEO_SG_Protection::$country_list,
            '#country_type_list' => \SEO_SG_Protection::$country_type_list,
        );

        $action = '';
        if (isset($_REQUEST['action'])) $action = htmlspecialchars(stripslashes(trim($_REQUEST['action'])));

        if ($action == 'EnableDisable_backend_protection') {
            $params = \SEO_SG_Protection::Get_Params(array('protection_backend'));
            \SEO_SG_Protection::Set_Params(array('protection_backend' => round(1 - $params['protection_backend'])));

            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();

            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
        }

        if ($action == 'Save_backend_params') {
            $data = array();
            if (isset($_POST['backend_ip_list'])) {
                $data['backend_ip_list'] = htmlspecialchars(stripslashes(trim($_POST['backend_ip_list'])));
            }
            if (isset($_POST['backend_ip_list_allow'])) {
                $data['backend_ip_list_allow'] = htmlspecialchars(stripslashes(trim($_POST['backend_ip_list_allow'])));
            }
            if (isset($_POST['country_list'])) {
                $data['backend_country_list'] = $_POST['country_list'];
            } else {
                $data['backend_country_list'] = array();
            }

            if (!$this->isPRO && count($data['backend_country_list']) > 5) {
                $data['backend_country_list'] = array_slice($data['backend_country_list'], 0, 5);
                $message_data = array(
                    'type' => 'info',
                    'header' => 'Free version limits',
                    'message' => 'Limit is 5 countries. Please upgrade.<br><b>For all websites with our <a href="https://www.siteguarding.com/en/antivirus-site-protection" target="_blank">PRO Antivirus plugin</a>, we provide with free license.</b>',
                    'button_text' => 'Upgrade',
                    'button_url' => 'https://www.siteguarding.com/en/buy-extention/drupal-geo-website-protection?domain=' . urlencode($GLOBALS['base_url']),
                    'help_text' => ''
                );

                $variables['#print_icon_data'] = $message_data;

                //update data
                $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
                $variables['#params']['backend_country_list'] = (isset($variables['#params']['backend_country_list'])) ? json_decode($variables['#params']['backend_country_list'], true) : '';
                //update data

                return $variables;
            }

            $data['backend_country_list'] = json_encode($data['backend_country_list']);

            $message_data = array(
                'type' => 'info',
                'header' => '',
                'message' => 'Backend settings saved',
                'button_text' => '',
                'button_url' => '',
                'help_text' => ''
            );

            $variables['#print_icon_data'] = $message_data;

            \SEO_SG_Protection::Set_Params($data);
            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();


            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
            $variables['#params']['backend_country_list'] = (isset($variables['#params']['backend_country_list'])) ? json_decode($variables['#params']['backend_country_list'], true) : '';
            //update data

        }

        return $variables;
    }

    public function georedir()
    {
        $variables = array(
            '#theme' => 'sggeoprotection_georedir',
            '#country_list' => \SEO_SG_Protection::$country_list,
            '#print_icon_data' => array(),
            '#params' => \SEO_SG_Protection::Get_Params(array()),
        );

        $action = '';
        if (isset($_REQUEST['action'])) $action = htmlspecialchars(stripslashes(trim($_REQUEST['action'])));

        if ($action == 'save_redirect_params') {
            $data['redirects'] = array();
            $redirects = $_POST['redirect'];
            if (is_array($redirects)) {
                foreach ($redirects as $redirectCountryCode => $redirectURL) {
                    $data['redirects'][htmlspecialchars(stripslashes(trim($redirectCountryCode)))] = htmlspecialchars(stripslashes(trim($redirectURL)));
                }
            }

            $data['redirects'] = array_filter($data['redirects']);

            if (!$this->isPRO && count($data['redirects']) > 10) {
                $data['redirects'] = array_slice($data['redirects'], 0, 10);
                $message_data = array(
                    'type' => 'info',
                    'header' => 'Free version limits',
                    'message' => 'Limit is 10 countries. Please upgrade.<br><b>For all websites with our <a href="https://www.siteguarding.com/en/antivirus-site-protection" target="_blank">PRO Antivirus plugin</a>, we provide with free license.</b>',
                    'button_text' => 'Upgrade',
                    'button_url' => 'https://www.siteguarding.com/en/buy-extention/drupal-geo-website-protection?domain=' . urlencode($GLOBALS['base_url']),
                    'help_text' => ''
                );

                $variables['#print_icon_data'] = $message_data;

                //update data
                $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
                $variables['#params']['redirects'] = (isset($variables['#params']['redirects'])) ? json_decode($variables['#params']['redirects'], true) : '';
                //update data

                return $variables;
            }

            $data['redirects'] = json_encode($data['redirects']);

            $message_data = array(
                'type' => 'info',
                'header' => '',
                'message' => 'GEO redirect settings saved',
                'button_text' => '',
                'button_url' => '',
                'help_text' => ''
            );

            $variables['#print_icon_data'] = $message_data;

            \SEO_SG_Protection::Set_Params($data);

            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();

            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
            $variables['#params']['redirects'] = (isset($variables['#params']['redirects'])) ? json_decode($variables['#params']['redirects'], true) : '';
            //update data

        }

        return $variables;
    }

    public function logs()
    {
        return [
            '#theme' => 'sggeoprotection_logs',
            '#latest_records_array' => \SEO_SG_Protection::GetLatestRecords(50),
            '#country_list' => \SEO_SG_Protection::$country_list,
        ];
    }

    public function custstyle()
    {
        $variables = array(
            '#theme' => 'sggeoprotection_custstyle',
            '#isPro' => $this->isPRO,
            '#params' => $this->params,
            '#baseurl' => $GLOBALS['base_url'],
            '#print_icon_data' => array(),
        );

        $action = '';
        if (isset($_REQUEST['action'])) $action = htmlspecialchars(stripslashes(trim($_REQUEST['action'])));

        if ($action == 'Customization_save') {
            $data = array();
            if (!$this->isPRO) {
                $data['custom_status'] = 0;

                $message_data = array(
                    'type' => 'info',
                    'header' => '',
                    'message' => 'Settings not saved. Please <a href="https://www.siteguarding.com/en/buy-extention/drupal-geo-website-protection" target="_blank">Get Full Version</a> to unlock all the features',
                    'button_text' => '',
                    'button_url' => '',
                    'help_text' => ''
                );

                $variables['#print_icon_data'] = $message_data;
                return $variables;

            } else {
                if (isset($_POST['custom_status'])) $data['custom_status'] = intval($_POST['custom_status']);
                if (isset($_POST['logo_url'])) $data['logo_url'] = htmlspecialchars(stripslashes(trim($_POST['logo_url'])));
                if (isset($_POST['text_1'])) $data['text_1'] = htmlspecialchars(stripslashes(trim($_POST['text_1'])));
                if (isset($_POST['text_2'])) $data['text_2'] = htmlspecialchars(stripslashes(trim($_POST['text_2'])));
                if (isset($_POST['hide_ipinfo'])) $data['hide_ipinfo'] = intval($_POST['hide_ipinfo']);
                else $data['hide_ipinfo'] = 0;
                if (isset($_POST['hide_debug'])) $data['hide_debug'] = intval($_POST['hide_debug']);
                else $data['hide_debug'] = 0;

                $message_data = array(
                    'type' => 'info',
                    'header' => '',
                    'message' => 'Settings not saved. Please <a href="https://www.siteguarding.com/en/buy-extention/drupal-geo-website-protection" target="_blank">Get Full Version</a> to unlock all the features',
                    'button_text' => '',
                    'button_url' => '',
                    'help_text' => ''
                );

                $variables['#print_icon_data'] = $message_data;

            }

            \SEO_SG_Protection::Set_Params($data);
            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();

            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
            $variables['#isPro'] = \SEO_SG_Protection::CheckIfPRO();
            //update data
        }

        return $variables;
    }

    public function settings()
    {
        $variables = array(
            '#theme' => 'sggeoprotection_settings',
            '#isPro' => $this->isPRO,
            '#print_icon_data' => array(),
            '#params' => $this->params,
            '#geodebug_flag' => (is_file(DRUPAL_ROOT . DIRECTORY_SEPARATOR . 'geodebug.txt')) ? true : false,
            '#pathdebugfile' => DRUPAL_ROOT . DIRECTORY_SEPARATOR . 'geodebug.txt',
            '#livechatimagepath' => $GLOBALS['base_url'] . DIRECTORY_SEPARATOR . drupal_get_path('module', 'sggeoprotection') . DIRECTORY_SEPARATOR . 'images' . DIRECTORY_SEPARATOR . 'livechat.png',

        );

        $action = '';
        if (isset($_REQUEST['action'])) $action = htmlspecialchars(stripslashes(trim($_REQUEST['action'])));

        if ($action == 'Save_Settings') {
            $data = array();
            if (isset($_POST['registration_code'])) $data['registration_code'] = htmlspecialchars(stripslashes(trim($_POST['registration_code'])));
            if (isset($_POST['protection_by'])) {
                $data['protection_by'] = intval($_POST['protection_by']);
            } else {
                $data['protection_by'] = 0;
            }
            if (!$this->isPRO) $data['protection_by'] = 1;

            $message_data = array(
                'type' => 'info',
                'header' => '',
                'message' => 'Settings saved',
                'button_text' => '',
                'button_url' => '',
                'help_text' => ''
            );

            $variables['#print_icon_data'] = $message_data;

            \SEO_SG_Protection::Set_Params($data);
            \SEO_SG_Protection::CreateSettingsFile();
            \SEO_SG_Protection::CheckWPConfig_file();

            //update data
            $variables['#params'] = \SEO_SG_Protection::Get_Params(array());
            $variables['#isPro'] = \SEO_SG_Protection::CheckIfPRO();
            //update data

        }

        return $variables;
    }

    public function load_geo_to_sql()
    {
        $geo_db_array = array();

        foreach (glob(dirname(dirname(dirname(__FILE__))) . DIRECTORY_SEPARATOR . "geo_base_*.db") as $filename) {
            $geo_db_array[] = $filename;
        }

        if (count($geo_db_array) > 0) {
            $params = \SEO_SG_Protection::Get_Params(array('geo_update_progress'));

            if (intval($params['geo_update_progress']) == 0) {
                \SEO_SG_Protection::Add_IP_adresses(true);
            }
        }
    }
}